@@ -1,44 +1,19 @@
-import asyncio
 import os
+import re
 import sys
-from functools import wraps
-from typing import Any, Callable, Optional
+from concurrent.futures import ThreadPoolExecutor
+from typing import Any
 
 import click
-from aiohttp import ClientResponse, ClientSession
-from bs4 import BeautifulSoup, ResultSet, Tag
+import git
+from git.exc import GitCommandError
 from yaml import Loader, dump, load
 
 
-def run_async(f: Callable) -> Callable:
-    @wraps(f)
-    def wrapper(*args, **kwargs) -> Any:
-        return asyncio.run(f(*args, **kwargs))
-
-    return wrapper
-
-
 def _colorize(text: str, color: str) -> str:
     return click.style(str(text), fg=color)
 
 
-async def _get_response(url: str, session: ClientSession) -> ClientResponse:
-    resp: ClientResponse = await session.get(
-        url,
-        headers={
-            "User-Agent": "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0"
-        },
-        ssl=False,
-    )
-    resp.raise_for_status()
-    return resp
-
-
-async def _get_html(url: str, session: ClientSession) -> BeautifulSoup:
-    response: ClientResponse = await _get_response(url, session)
-    return BeautifulSoup(await response.text(), "html.parser")
-
-
 def _read_yaml_file(file_path: str) -> str:
     with open(file_path) as f:
         content: str = f.read()
@@ -60,73 +35,46 @@ def _get_target_tag(tags: list, all_versions: bool) -> str:
     return tags[0]
 
 
-def _parse_github_tags(url: str, soup: BeautifulSoup) -> list:
-    tag_versions: list = []
-    tags: Optional[ResultSet] = soup.find_all(
-        "div", class_="Box-row position-relative d-flex"
-    )
-    if not tags:
-        raise Exception(f"No tags found for repo: {url}")
-    for tag in tags:
-        v: Optional[Tag] = tag.find("a")
-        if not v:
-            raise Exception(f"No tags found for repo: {url}")
-        tag_versions.append(v.text)
-    return tag_versions
-
-
-def _parse_gitlab_tags(url: str, soup: BeautifulSoup) -> list:
-    tag_versions: list = []
-    tags: Optional[ResultSet] = soup.find_all(
-        "li",
-        class_="flex-row js-tag-list gl-white-space-normal! gl-align-items-flex-start!",
-    )
-    if not tags:
-        raise Exception(f"No tags found for repo: {url}")
-    for tag in tags:
-        v: Optional[Tag] = tag.find("a")
-        if not v:
-            raise Exception(f"No tags found for repo: {url}")
-        tag_versions.append(v.text)
-    return tag_versions
-
-
-def _parse_tags(repo: dict, soup: BeautifulSoup) -> list:
+def _parse_tags(repo: dict) -> list:
     url: str = repo["repo"]
-    if "github.com" in url:
-        tag_versions = _parse_github_tags(url, soup)
-    elif "gitlab.com" in url:
-        tag_versions = _parse_gitlab_tags(url, soup)
-    else:
-        raise Exception(f"Unsupported provider for repo: {url}")
-    return tag_versions
+    try:
+        remote_tags: list = (
+            git.cmd.Git().ls_remote("--tags", url, sort="v:refname").split("\n")
+        )
+        tags: list = []
+        for tag in remote_tags:
+            parsed_tag: str = re.split(r"\t+", tag)[1]
+            if parsed_tag.endswith("^{}"):
+                continue
+            parsed_tag = parsed_tag.replace("refs/tags/", "")
+            tags.append(parsed_tag)
+        return tags
+    except GitCommandError:
+        raise Exception(f"No tags found for repo: {url}")
 
 
-async def run(dry_run: bool, all_versions: bool, verbose: bool) -> None:
-    session: ClientSession = ClientSession()
+def run(dry_run: bool, all_versions: bool, verbose: bool) -> None:
+    os.environ["GIT_TERMINAL_PROMPT"] = "0"
     try:
         file_path: str = os.path.join(os.getcwd(), ".pre-commit-config.yaml")
         yaml_str: str = _read_yaml_file(file_path)
         data: Any = load(yaml_str, Loader)
-        skip_repo: list = []
-        tasks: list = []
         no_diff: list = []
         diff: list = []
 
+        with ThreadPoolExecutor(max_workers=10) as pool:
+            tasks: list = []
+            for i in range(len(data["repos"])):
+                rep: dict = data["repos"][i]
+                tasks.append(pool.submit(_parse_tags, rep))
+
         for i, repository in enumerate(data["repos"]):
             if not repository["repo"].startswith("http"):
-                skip_repo.append(i)
-                continue
-            tasks.append(_get_html(f"{repository['repo']}/tags", session))
-
-        results: Any = await asyncio.gather(*tasks)
-
-        for i in range(len(data["repos"])):
-            if i in skip_repo:
                 continue
             rep: dict = data["repos"][i]
             hook: str = rep["repo"].split("/")[-1]
-            tag_versions: list = _parse_tags(rep, results[i])
+            tag_versions: list = tasks[i].result()
+            tag_versions.reverse()
             target_ver: str = _get_target_tag(tag_versions, all_versions)
             if rep["rev"] != target_ver:
                 diff.append(
@@ -152,9 +100,6 @@ async def run(dry_run: bool, all_versions: bool, verbose: bool) -> None:
     except Exception as ex:
         sys.exit(str(ex))
 
-    finally:
-        await session.close()
-
 
 @click.command(context_settings=dict(help_option_names=["-h", "--help"]))
 @click.option(
@@ -181,9 +126,8 @@ async def run(dry_run: bool, all_versions: bool, verbose: bool) -> None:
     default=False,
     help="Display complete output",
 )
-@run_async
-async def cli(dry_run: bool, all_versions: bool, verbose: bool):
-    await run(dry_run, all_versions, verbose)
+def cli(dry_run: bool, all_versions: bool, verbose: bool):
+    run(dry_run, all_versions, verbose)
 
 
 if __name__ == "__main__":
